"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var user_entity_1 = require("./entities/user.entity");
var ufs_1 = require("./ufs");
var ConfigBuilder = /** @class */ (function () {
    function ConfigBuilder(basePath) {
        this.config = {
            disabledDb: false,
            disabledGraphQL: false,
            db: {
                type: "mysql",
                host: "localhost",
                port: 8889,
                username: "root",
                password: "root",
                database: "koa_typescript2",
                entities: [basePath + "/entities/*.entity.js"],
                synchronize: true,
                logging: false
            },
            publicFolders: [basePath + "/public"],
            viewFolders: [basePath + "/views"],
            translationFolders: [basePath + "/locale"],
            middlewaresFolders: [basePath + "/middlewares"],
            controllersFolders: [basePath + "/controllers"],
            migrationsFolders: [basePath + "/migrations"],
            disableMigrations: false,
            sessionSecret: "session_secret",
            sessionStore: null,
            tokenSecret: "token_secret",
            mailer: {
                sender: 'Lynx Framework <lynx.framework@fakemail.com>',
                host: '',
                port: 587,
                secure: false,
                auth: {
                    user: '',
                    pass: ''
                }
            },
            defaultLanguage: "it",
            uploadPath: basePath + "/../uploads",
            cachePath: basePath + "/../cache",
            ufs: new ufs_1.LocalUFS(),
            onDatabaseInit: function () { },
            chachingImages: false
        };
    }
    ConfigBuilder.prototype.setPublicFolders = function (folders) {
        this.config.publicFolders = folders;
        return this;
    };
    ConfigBuilder.prototype.setViewFolders = function (folders) {
        this.config.viewFolders = folders;
        return this;
    };
    ConfigBuilder.prototype.setTranslationFolders = function (folders) {
        this.config.translationFolders = folders;
        return this;
    };
    ConfigBuilder.prototype.setMiddlewaresFolders = function (folders) {
        this.config.middlewaresFolders = folders;
        return this;
    };
    ConfigBuilder.prototype.setControllersFolders = function (folders) {
        this.config.controllersFolders = folders;
        return this;
    };
    ConfigBuilder.prototype.setMigrationsFolders = function (folders) {
        this.config.migrationsFolders = folders;
        return this;
    };
    ConfigBuilder.prototype.setSessionSecret = function (secret) {
        this.config.sessionSecret = secret;
        return this;
    };
    ConfigBuilder.prototype.setTokenSecret = function (secret) {
        this.config.tokenSecret = secret;
        return this;
    };
    ConfigBuilder.prototype.setSessionStore = function (store) {
        this.config.sessionStore = store;
        return this;
    };
    ConfigBuilder.prototype.setUploadPath = function (path) {
        this.config.uploadPath = path;
        return this;
    };
    ConfigBuilder.prototype.setCachePath = function (path) {
        this.config.cachePath = path;
        return this;
    };
    ConfigBuilder.prototype.setDefaultLanguage = function (language) {
        this.config.defaultLanguage = language;
        return this;
    };
    ConfigBuilder.prototype.setEntitiesFolders = function (folders) {
        this.config.db.entities = folders;
        return this;
    };
    ConfigBuilder.prototype.setDatabaseType = function (type) {
        this.config.disabledDb = false;
        this.config.db.type = type;
        return this;
    };
    ConfigBuilder.prototype.setDatabaseHost = function (host) {
        this.config.disabledDb = false;
        this.config.db.host = host;
        return this;
    };
    ConfigBuilder.prototype.setDatabasePort = function (port) {
        this.config.disabledDb = false;
        this.config.db.port = port;
        return this;
    };
    ConfigBuilder.prototype.setDatabaseLogin = function (username, password) {
        this.config.disabledDb = false;
        this.config.db.username = username;
        this.config.db.password = password;
        return this;
    };
    ConfigBuilder.prototype.setDatabase = function (database) {
        this.config.disabledDb = false;
        this.config.db.database = database;
        return this;
    };
    ConfigBuilder.prototype.disableDB = function () {
        this.config.disabledDb = true;
        return this;
    };
    ConfigBuilder.prototype.disableMigration = function () {
        this.config.disableMigrations = true;
        return this;
    };
    ConfigBuilder.prototype.enableMigration = function () {
        this.config.disableMigrations = false;
        return this;
    };
    ConfigBuilder.prototype.disableGraphQL = function () {
        this.config.disabledGraphQL = true;
        return this;
    };
    ConfigBuilder.prototype.setMailerSender = function (address) {
        this.config.mailer.sender = address;
        return this;
    };
    ConfigBuilder.prototype.setMailerAuth = function (user, password) {
        this.config.mailer.auth.user = user;
        this.config.mailer.auth.pass = password;
        return this;
    };
    ConfigBuilder.prototype.setMailerServer = function (host, port, secure) {
        this.config.mailer.host = host;
        this.config.mailer.port = port;
        this.config.mailer.secure = secure;
        return this;
    };
    ConfigBuilder.prototype.setCustomUserEntity = function (hasCustom) {
        user_entity_1.setSkipSync(!hasCustom);
        return this;
    };
    ConfigBuilder.prototype.setJsonLimit = function (limit) {
        this.config.jsonLimit = limit;
        return this;
    };
    ConfigBuilder.prototype.setUFS = function (ufs) {
        this.config.ufs = ufs;
        return this;
    };
    ConfigBuilder.prototype.setOnDatabaseInit = function (cb) {
        this.config.onDatabaseInit = cb;
        return this;
    };
    ConfigBuilder.prototype.enableCachingImages = function () {
        this.config.chachingImages = true;
        return this;
    };
    ConfigBuilder.prototype.build = function () {
        return this.config;
    };
    return ConfigBuilder;
}());
exports.ConfigBuilder = ConfigBuilder;
