"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var pluralize = require("pluralize");
var typeorm_1 = require("typeorm");
var graphql_tools_1 = require("graphql-tools");
var graphqlFields = require("graphql-fields");
var logger_1 = require("../logger");
function generateSchema(entitiesPaths) {
    var _this = this;
    var data = [];
    for (var _i = 0, entitiesPaths_1 = entitiesPaths; _i < entitiesPaths_1.length; _i++) {
        var path = entitiesPaths_1[_i];
        if (path.endsWith("/*.entity.js")) {
            path = path.substring(0, path.length - "/*.entity.js".length);
        }
        if (!fs.existsSync(path)) {
            continue;
        }
        var files = fs.readdirSync(path);
        for (var index in files) {
            if (files[index].endsWith("ts"))
                continue;
            var entity = require(path + "/" + files[index]).default;
            if (!entity) {
                continue;
            }
            if (entity.graphQL && entity.graphQL.hasGraphQL) {
                data.push({ entity: entity, meta: entity.graphQL });
            }
        }
    }
    var queries = "";
    var mutators = "";
    var schemas = "";
    var resolvers = { Query: {}, Mutation: {} };
    var dbNames = [];
    var _loop_1 = function (d) {
        var inputType = "";
        var cName = d.meta.className.toLowerCase();
        if (dbNames.indexOf(d.meta.className) != -1) {
            logger_1.logger.warn("The entity " +
                cName +
                " reference to " +
                d.meta.className +
                " tables, which is already mapped!");
            return "continue";
        }
        dbNames.push(d.meta.className);
        var queryName = pluralize(cName);
        queryName =
            "findAll" +
                queryName.substring(0, 1).toUpperCase() +
                queryName.substring(1);
        schemaType = "type " + d.meta.className + " {\n";
        queries += "\t" + queryName + "(";
        inputType = "input " + d.meta.className + "Input {\n";
        for (var i = 0; i < d.meta.fields.length; i++) {
            var field = d.meta.fields[i];
            schemaType += "\t" + field.name + ": " + mapType(field.type) + "\n";
            if (field.type.startsWith("[")) {
                continue;
            }
            if (field.writable && field.type != TYPE_ID) {
                inputType += "\t" + field.name + ": " + mapType(field.type) + "\n";
            }
            queries += field.name + ": " + mapType(field.type) + ", ";
            if (field.type == "String") {
                queries += field.name + "Like: " + mapType(field.type) + ", ";
            }
        }
        queries += " " + OFFSET + ": Int, " + LIMIT + ": Int";
        schemaType += "}\n";
        queries += "): [" + d.meta.className + "] \n";
        inputType += "}\n";
        schemaType += inputType;
        mutators += "\tcreate" + d.meta.className + "(input: " + d.meta.className + "Input): " + d.meta.className + "\n";
        mutators += "\tupdate" + d.meta.className + "(id: ID!, input: " + d.meta.className + "Input): " + d.meta.className + "\n";
        mutators += "\tdelete" + d.meta.className + "(id: ID!): " + d.meta.className + "\n";
        schemas += schemaType;
        resolvers.Query[queryName] = function (_, args, __, info) { return __awaiter(_this, void 0, void 0, function () {
            var request, qb, _i, _a, field, f, arg, p, key;
            return __generator(this, function (_b) {
                request = graphqlFields(info);
                qb = typeorm_1.getRepository(d.entity)
                    .createQueryBuilder(cName)
                    .select();
                for (_i = 0, _a = d.meta.fields; _i < _a.length; _i++) {
                    field = _a[_i];
                    if (field.type.indexOf("[") == -1) {
                        continue;
                    }
                    if (!request.hasOwnProperty(field.name)) {
                        continue;
                    }
                    f = field.type.replace("[", "").replace("]", "");
                    qb = qb.leftJoinAndSelect(cName + "." + field.name, f);
                }
                qb = qb.where("true");
                for (arg in args) {
                    if (isReservedArgs(arg)) {
                        continue;
                    }
                    p = {};
                    p[arg] = args[arg];
                    key = arg;
                    if (arg.endsWith("Like")) {
                        key = arg.substring(0, arg.length - 4);
                        p[arg] = "%" + p[arg] + "%";
                        qb = qb.andWhere(cName + "." + key + " LIKE :" + arg, p);
                    }
                    else {
                        qb = qb.andWhere(cName + "." + key + " =  :" + arg, p);
                    }
                }
                if (args[OFFSET]) {
                    qb.offset(args[OFFSET]);
                }
                if (args[LIMIT]) {
                    qb.limit(args[LIMIT]);
                }
                return [2 /*return*/, qb.getMany()];
            });
        }); };
        resolvers.Mutation["create" + d.meta.className] = function (_, args, __, ___) { return __awaiter(_this, void 0, void 0, function () {
            var input, key, obj;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        input = {};
                        for (key in args.input) {
                            input[key] = args.input[key];
                        }
                        return [4 /*yield*/, typeorm_1.getRepository(d.entity).create()];
                    case 1:
                        obj = _a.sent();
                        return [4 /*yield*/, typeorm_1.getRepository(d.entity).merge(obj, input)];
                    case 2:
                        obj = _a.sent();
                        return [4 /*yield*/, obj.save()];
                    case 3:
                        _a.sent();
                        return [2 /*return*/, obj];
                }
            });
        }); };
        resolvers.Mutation["update" + d.meta.className] = function (_, args, __, ___) { return __awaiter(_this, void 0, void 0, function () {
            var id, obj, key;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        id = args.id;
                        return [4 /*yield*/, typeorm_1.getRepository(d.entity).findOne(id)];
                    case 1:
                        obj = (_a.sent());
                        for (key in args.input) {
                            obj[key] = args.input[key];
                        }
                        return [4 /*yield*/, obj.save()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, obj];
                }
            });
        }); };
        resolvers.Mutation["delete" + d.meta.className] = function (_, args, __, ___) { return __awaiter(_this, void 0, void 0, function () {
            var obj;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, typeorm_1.getRepository(d.entity).findOne(args.id)];
                    case 1:
                        obj = (_a.sent());
                        return [4 /*yield*/, typeorm_1.getRepository(d.entity).delete(args.id)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, obj];
                }
            });
        }); };
    };
    var schemaType;
    for (var _a = 0, data_1 = data; _a < data_1.length; _a++) {
        var d = data_1[_a];
        _loop_1(d);
    }
    queries = "type Query {\n" + queries + "}\n";
    mutators = "type Mutation {\n" + mutators + "}\n";
    var typeDefs = queries + "\n" + mutators + "\n" + schemas;
    return graphql_tools_1.makeExecutableSchema({
        typeDefs: typeDefs,
        resolvers: resolvers
    });
}
exports.generateSchema = generateSchema;
function mapType(type) {
    var isArray = false;
    if (type.endsWith("[]")) {
        isArray = true;
        type = type.replace("[]", "");
    }
    switch (type) {
        case "Number":
            return toTypeArray("Int", isArray);
    }
    return toTypeArray(type, isArray);
}
function toTypeArray(type, isArray) {
    if (isArray) {
        return "[" + type + "]";
    }
    return type;
}
var TYPE_ID = "ID";
var OFFSET = "_offset";
var LIMIT = "_limit";
var reservedArgs = [OFFSET, LIMIT];
function isReservedArgs(arg) {
    for (var _i = 0, reservedArgs_1 = reservedArgs; _i < reservedArgs_1.length; _i++) {
        var res = reservedArgs_1[_i];
        if (arg == res) {
            return true;
        }
    }
    return false;
}
